 #
 #   $Id: Revision.pm,v 1.1 2004/05/14 23:52:22 zarahg Exp $
 #
 #   This  file  is  part of the  PodWiki  web-based authoring tool.
 #
 #   By  accessing  this software,  PodWiki,  you are  duly informed
 #   of and  agree to be  bound  by the  conditions  described below
 #   in this notice:
 #
 #   This software product, PodWiki,  is developed by  Thomas Linden
 #   and   copyrighted  (C) 2003-2004   by  Thomas Linden,  with all
 #   rights reserved.
 #
 #   There is  no charge for  PodWiki software. You can redistribute
 #   it and/or modify it under  the terms of the  GNU General Public
 #   License, which is incorporated by reference herein.
 #
 #   PodWiki is distributed WITHOUT ANY WARRANTY,IMPLIED OR EXPRESS,
 #   OF  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
 #   the  use of it will not infringe on any third party's intellec-
 #   tual property rights.
 #
 #   You  should  have  received a  copy  of  the GNU General Public
 #   License along with PodWiki.  Copies  can also be obtained from:
 #
 #     http://www.gnu.org/copyleft/gpl.html
 #
 #   or by writing to:
 #
 #     Free Software Foundation, Inc.
 #     59 Temple Place, Suite 330
 #     Boston, MA 02111-1307
 #     USA
 #
 #   Or contact:
 #
 #    "Thomas Linden" <tom@daemon.de>
 #
 #   Additional Copyrights:
 #
 #




=head1 NAME

PodWiki::Revision - RCS wrapper to hide RCS initialization and command line parameter weirdness.

=head1 SYNOPSIS

 use PodWiki::Revision;

 my $rev = new PodWiki::Revision(
                   file    => "SandBox",
                   bindir  => "/usr/bin",
                   workdir => "./data"
           );

 # initially checkin a file
 $rev->add("Initial Checkin By PodWiki Setup");

 # checkin a changed file creating a new revision
 $rev->commit("bugfix", "root", "Exp");

 # checkout a revision(default last)
 $rev->checkout("1.41");


=head1 DESCRIPTION

This simple wrapper exists to hide RCS details in PodWiki modules
and to centralize RCS handling.

=head1 METHODS

=cut



package PodWiki::Revision;

$PodWiki::Revision::VERSION = "1.00";

use lib qw(. lib);

use strict;
no strict "refs";
use Carp;
use Exporter;
use Rcs;

use vars qw(@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS $AUTOLOAD);

@ISA=qw(Exporter);
@EXPORT=qw();
@EXPORT_OK=qw();
@EXPORT_TAGS=();


=head2 new

Creates a new PodWiki::Revision object

Parameters must be given as hash.

Parameters:

=over

=item I<file>

Filename to work on.

=item I<bindir>

Where to find the RCS binaries (default: "/usr/bin").

=item I<workdir>

Where to store the ,v state files.

=back

Example:

 my $rev = new PodWiki::Revision(
                   file    => "SandBox",
                   bindir  => "/usr/bin",
                   workdir => "./data"
           );

=cut


sub new {
  my($this, %param) = @_;
  my $class = ref($this) || $this;
  my $self = \%param;
  bless($self, $class);

  if (! exists $self->{file}) {
    croak "Unable to initialize RCS without a file. Parameter 'file' required!\n";
  }

  # set defaults if not given
  if (! exists $self->{bindir}) {
    $self->{bindir} = "/usr/bin/";
  }
  if (! exists $self->{workdir}) {
    $self->{workdir} = "./data";
  }
  if (! exists $self->{debug}) {
    $self->{debug} = 0;
  }
  $self->{rcsdir} = $self->{workdir};

  # initialize RCS
  $self->{rcs} = new Rcs;
  $self->{rcs}->bindir($self->{bindir});
  $self->{rcs}->workdir($self->{workdir});
  $self->{rcs}->rcsdir($self->{rcsdir});
  $self->{rcs}->file($self->{file});

  $self->debug("::new(workdir => \"$self->{workdir}\", file => \"$self->{file}\")");

  # go ahead
  return $self;
}


=head2 add

Adds (initial checkin) a new file to the repository.

Parameters must be given as list.

Parameters:

 ($comment [, $login])

=over

=item I<$comment>

A string describing the changes.

=item I<$login>

The user who does the checkin.

=back

Example:

 $rev->add("Initial Checkin By PodWiki Setup", "root");

=cut

sub add {
  my($this, $comment, $login) = @_;

  $this->debug("::add($comment, $login)");

  my @opt = ('-u');

  if (defined $comment) {
    push @opt, "-m $comment";
  }

  if (defined $login) {
    push @opt, "-w $login"
  }

  $this->debug("   RCS: @opt");

  return $this->{rcs}->ci(@opt);
}



=head2 commit

Commits (checkin) a file to the repository creating a new revision.

Parameters must be given as list.

Parameters:

 ($comment [, $login [, $state] ])

=over

=item I<$comment>

A string describing the changes.

=item I<$login>

The user who does the checkin.

=item I<$state>

The state as to use for the RCS checkin.

=back

Example:

 $rev->commit("bugfix", "root", "Exp");

=cut

sub commit {
  my($this, $comment, $login, $state) = @_;

  $this->debug("::commit($comment, $login, $state)");

  my @opt = ('-u');

  if (defined $comment) {
    push @opt, "-m$comment";
  }

  if (defined $login) {
    push @opt, "-w$login"
  }

  if (defined $state) {
    push @opt, "-s$state"
  }

  $this->debug("   RCS: ". join(",",@opt));

  $this->{rcs}->ci(@opt);

  $this->debug("   LOCK: ". $this->{rcs}->lock);
}



=head2 checkout

Checks out a revision of a file. If no revision given it checks the latest one.

Parameters must be given as list.

Parameters:

 ([$revision])

=over

=item I<$revision>

A valid RCS revision number, e.g. "1.41".

=back

Example:

$rev->checkout();       # get latest
$rev->checkout("1.41"); # get 1.41

=cut

sub checkout {
  my($this, $revision) = @_;

  $this->debug("::checkout()");

  my @opt = (); # FIXME: page edit LOCK problem!

  if (defined $revision) {
    push @opt, "-r $revision";
  }

  $this->debug("   RCS: @opt");

  $this->{rcs}->co(@opt);

  $this->debug("   LOCK: ". $this->{rcs}->lock);
}



=head2 LOCK

Locks the last revision of a file.

Parameters: none.

=cut

sub LOCK {
  my($this) = @_;

  $this->debug("::LOCK()");

  my @opt = ('-l');

  $this->debug("   RCS: ". join(",",@opt));

  $this->{rcs}->co(@opt);
}


=head2 UNLOCK

Unlocks the last revision of a file.

Parameters: none.

=cut

sub UNLOCK {
  my($this) = @_;

  $this->debug("::UNLOCK()");

  my @opt = ('-u', '-M');

  $this->debug("   RCS: ". join(",",@opt));

  $this->{rcs}->rcs(@opt);

  # FIXME: how to do this with RCS? If I just do rcs -u -M it leaves
  #        the file writable which annoyes following RCS commands :-|
  system("chmod 444 $this->{workdir}/$this->{file}");
}



=head2 AUTOLOAD

You can always access the underlying RCS object just
by calling the original method using the required RCS
commandline flags.

This can be used if special parameters have to be given
or unwrapped stuff has to be done.

Example usage:

 $rev->ci('-l', "-s$input{rcs_state}", "-m $input{comment}", "-w$input{login}");

=cut


sub AUTOLOAD {
  #
  # wrapper directly to RCS
  my($this, @stuff) = @_;
  my $sub = $PodWiki::Revision::AUTOLOAD;  # get to know how we were called
  $sub =~ s/.*:://; # remove package name!

  $this->debug("->$sub(@stuff)");

  return $this->{rcs}->$sub(@stuff);
}

sub DESTROY {}

sub debug {
  my ($this, $msg) = @_;
  if ($this->{debug}) {
    print STDERR "PodWiki::Revision:  " . $msg . "\n";
  }
}

1;



=head1 SEE ALSO

rcs(1), ci(1), co(1), perl(1), L<C<PodWiki::Runner>>, L<C<Rcs>>


=head1 AUTHOR

Thomas Linden <tom@daemon.de>



=head1 COPYRIGHT

Copyright (c) 2004 by Thomas Linden.


=cut
